package org.eclipse.jdt.internal.compiler.apt.model;

package .tmp;

import java.lang.annotation.Annotation;
import java.lang.annotation.Inherited;
import java.util.Collections;
import java.util.List;
import java.util.Set;
import javax.lang.model.element.AnnotationMirror;
import javax.lang.model.element.Element;
import javax.lang.model.element.ElementKind;
import javax.lang.model.element.Modifier;
import javax.lang.model.element.Name;
import javax.lang.model.element.PackageElement;
import javax.lang.model.type.TypeMirror;
import org.eclipse.jdt.internal.compiler.apt.dispatch.BaseProcessingEnvImpl;
import org.eclipse.jdt.internal.compiler.apt.model.Factory;
import org.eclipse.jdt.internal.compiler.apt.model.IElementInfo;
import org.eclipse.jdt.internal.compiler.apt.model.NameImpl;
import org.eclipse.jdt.internal.compiler.lookup.AnnotationBinding;
import org.eclipse.jdt.internal.compiler.lookup.Binding;
import org.eclipse.jdt.internal.compiler.lookup.ReferenceBinding;

public abstract class ElementImpl implements Element, IElementInfo {
  public final BaseProcessingEnvImpl _env;
  
  public final Binding _binding;
  
  protected ElementImpl(BaseProcessingEnvImpl env, Binding binding) {
    this._env = env;
    this._binding = binding;
  }
  
  public TypeMirror asType() {
    return this._env.getFactory().newTypeMirror(this._binding);
  }
  
  protected abstract AnnotationBinding[] getAnnotationBindings();
  
  public final AnnotationBinding[] getPackedAnnotationBindings() {
    return Factory.getPackedAnnotationBindings(getAnnotationBindings());
  }
  
  public <A extends Annotation> A getAnnotation(Class<A> annotationClass) {
    Annotation annotation = this._env.getFactory().getAnnotation(getPackedAnnotationBindings(), annotationClass);
    if (annotation != null || getKind() != ElementKind.CLASS || annotationClass.getAnnotation(Inherited.class) == null)
      return (A)annotation; 
    org.eclipse.jdt.internal.compiler.apt.model.ElementImpl superClass = (org.eclipse.jdt.internal.compiler.apt.model.ElementImpl)this._env.getFactory().newElement((Binding)((ReferenceBinding)this._binding).superclass());
    return (superClass == null) ? null : superClass.<A>getAnnotation(annotationClass);
  }
  
  public List<? extends AnnotationMirror> getAnnotationMirrors() {
    return this._env.getFactory().getAnnotationMirrors(getPackedAnnotationBindings());
  }
  
  public <A extends Annotation> A[] getAnnotationsByType(Class<A> annotationType) {
    Annotation[] annotations = this._env.getFactory().getAnnotationsByType(Factory.getUnpackedAnnotationBindings(getPackedAnnotationBindings()), annotationType);
    if (annotations.length != 0 || getKind() != ElementKind.CLASS || annotationType.getAnnotation(Inherited.class) == null)
      return (A[])annotations; 
    org.eclipse.jdt.internal.compiler.apt.model.ElementImpl superClass = (org.eclipse.jdt.internal.compiler.apt.model.ElementImpl)this._env.getFactory().newElement((Binding)((ReferenceBinding)this._binding).superclass());
    return (superClass == null) ? (A[])annotations : superClass.<A>getAnnotationsByType(annotationType);
  }
  
  public Set<Modifier> getModifiers() {
    return Collections.emptySet();
  }
  
  public Name getSimpleName() {
    return (Name)new NameImpl(this._binding.shortReadableName());
  }
  
  public int hashCode() {
    return this._binding.hashCode();
  }
  
  public boolean equals(Object obj) {
    if (this == obj)
      return true; 
    if (obj == null)
      return false; 
    if (getClass() != obj.getClass())
      return false; 
    org.eclipse.jdt.internal.compiler.apt.model.ElementImpl other = (org.eclipse.jdt.internal.compiler.apt.model.ElementImpl)obj;
    if (this._binding == null) {
      if (other._binding != null)
        return false; 
    } else if (this._binding != other._binding) {
      return false;
    } 
    return true;
  }
  
  public String toString() {
    return this._binding.toString();
  }
  
  public String getFileName() {
    return null;
  }
  
  abstract PackageElement getPackage();
  
  public boolean hides(Element hidden) {
    return false;
  }
}
